﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;

namespace HeuristicLab.EvolutionTracking.Views {
  [View("GenealogyGraph")]
  [Content(typeof(GenealogyGraph), IsDefaultView = true)]
  public sealed partial class GenealogyGraphView : ItemView {
    public new GenealogyGraph Content {
      get { return (GenealogyGraph)base.Content; }
      set { base.Content = value; }
    }

    public GenealogyGraphView()
      : base() {
      InitializeComponent();
      // set button icons here because if set in the designer file, they get overwritten
      this.highlightAllButton.Image = Common.Resources.VSImageLibrary.Gradient;
      this.simpleLineagesCheckBox.Image = Common.Resources.VSImageLibrary.ArrowDown;
      this.lockGenealogyCheckBox.Image = Common.Resources.VSImageLibrary.ProtectForm;
    }

    protected override void DeregisterContentEvents() {
      // TODO: Deregister your event handlers here
      Content.GraphUpdated -= GenealogyGraphUpdated;
      base.DeregisterContentEvents();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      // TODO: Register your event handlers here
      Content.GraphUpdated += GenealogyGraphUpdated;
    }

    #region Event Handlers (Content)
    // TODO: Put event handlers of the content here
    protected override void OnContentChanged() {
      base.OnContentChanged();
      genealogyGraphChart.GenealogyGraph = Content ?? null;
    }
    private void Content_GraphChanged(object sender, EventArgs e) {
    }
    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      genealogyGraphChart.Enabled = Content != null;
    }
    #endregion

    #region Event Handlers (child controls)
    private void GenealogyGraphUpdated(object sender, EventArgs args) {
      genealogyGraphChart.GenealogyGraph = Content;
    }
    void MatchNodesAndRepaint() {
      genealogyGraphChart.Chart.UpdateEnabled = false;
      genealogyGraphChart.ClearPrimitives(); // clear node colors

      genealogyGraphChart.Chart.UpdateEnabled = true;
      genealogyGraphChart.Chart.EnforceUpdate();
    }

    private void treeChart_SymbolicExpressionTreeNodeDoubleClicked(object sender, MouseEventArgs e) {
    }
    #endregion
    private void simpleLineagesCheckBox_CheckedChanged(object sender, EventArgs e) {
      genealogyGraphChart.SimpleLineages = simpleLineagesCheckBox.Checked;
    }

    private void highlightAllButton_Click(object sender, EventArgs e) {
    }

    private void lockGenealogyCheckBox_CheckedChanged(object sender, EventArgs e) {
      genealogyGraphChart.LockGenealogy = lockGenealogyCheckBox.Checked;
    }

    private void highlightRootParentsButton_Click(object sender, EventArgs e) {
      var nodes = genealogyGraphChart.GenealogyGraph.Nodes.Where(n => n.InArcs != null && n.InArcs.Count == 2).Select(n => (GenealogyGraphNode)n.InArcs[0].Source).ToList();
      genealogyGraphChart.HighlightNodes(nodes);
    }

    private void highlightSecondaryParentsButton_Click(object sender, EventArgs e) {
      var nodes = genealogyGraphChart.GenealogyGraph.Nodes.Where(n => n.InArcs != null && n.InArcs.Count == 2).Select(n => (GenealogyGraphNode)n.InArcs[1].Source).ToList();
      genealogyGraphChart.HighlightNodes(nodes);
    }
  }
}
